"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var enzyme_1 = require("enzyme");
require("@reactivex/rxjs");
var __1 = require("..");
var rx = require("../xs/rx");
var rxjs_1 = require("@reactivex/rxjs");
require("@reactivex/rxjs/dist/cjs/add/observable/combineLatest");
require("@reactivex/rxjs/dist/cjs/add/operator/filter");
var xtests_1 = require("../xtests");
var forms_1 = require("../forms");
var compose = function (f, g) { return function (x) { return f(g(x)); }; };
var CounterView = function (props) { return (React.createElement("div", { className: "counter-view" },
    React.createElement("span", { className: "count" }, props.count))); };
CounterView.defaultProps = { count: 0, count1: 1, count2: 0 };
var mountx = compose(enzyme_1.mount, function (y) { return React.createFactory(__1.X)({ x: rx }, y); });
var fantasyX = __1.fromPlan(function (intent$) {
    return {
        update$: intent$.map(function (intent) {
            switch (intent.type) {
                case 'inc':
                    return function (state) { return ({ count: state.count + 1 }); };
                case 'dec':
                    return function (state) { return ({ count: state.count - 1 }); };
                default:
                    return function (state) { return state; };
            }
        }),
        actions: {
            inc: function () { return ({ type: 'inc' }); },
            dec: function () { return ({ type: 'dec' }); },
        }
    };
});
describe('actions', function () {
    var Counter, counterWrapper, counter, t, counterView, actions;
    describe('basic', function () {
        beforeEach(function () {
            Counter = fantasyX.apply(CounterView);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(CounterView);
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('add intent to intent$ and go through sink$', function () {
            return t
                .do([
                actions.inc,
                actions.inc,
                actions.inc,
            ])
                .collect(counter)
                .then(function (x) { return expect(x.count).toBe(3); });
        });
    });
    describe('map', function () {
        beforeEach(function () {
            Counter = fantasyX.map(function (a) { return ({ count1: (a.count1 || 1) * 2 }); }).concat(fantasyX.map(function (a) { return ({ count2: (a.count2 || 0) + 2 }); })).apply(CounterView);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(CounterView);
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('inc will + 1 then * 2', function () {
            return t
                .do([
                actions.inc,
                actions.inc,
                actions.inc,
            ])
                .collect(counter)
                .then(function (x) {
                /* -(count+1) (count1*2)-(count+1) (count1*2)-(count+1) (count1*2)-> */
                /* -(count+1) (count2+2)-(count+1) (count2+2)-(count+1) (count2+2)-> */
                expect(x).toEqual({ "count": 6, "count1": 8, "count2": 6 });
            });
        });
    });
    describe('bimap', function () {
        beforeEach(function () {
            Counter = fantasyX.bimap(function () { return ({
                double: function () { return ({ type: 'double' }); },
                inc: function () { return ({ type: 'inc' }); }
            }); }, function (a) { return ({ count: (a.count || 0) * 2 }); })
                .apply(CounterView);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(CounterView);
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('inc will + 1 then * 2, otherwise * 2', function () {
            return t
                .do([
                actions.inc,
                actions.double,
                actions.inc,
            ])
                .collect(counter)
                .then(function (x) { return expect(x.count).toBe(10); });
        });
    });
    describe('lift', function () {
        beforeEach(function () {
            Counter = __1.lift(function (a) { return ({ count: (a.count || 0) * 2 }); })(fantasyX).apply(CounterView);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(CounterView);
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('inc will + 1 then * 2', function () {
            return t
                .do([
                actions.inc,
                actions.inc,
                actions.inc,
            ])
                .collect(counter)
                .then(function (x) { return expect(x.count).toBe(14); });
        });
    });
    describe('concat', function () {
        var fantasyXB;
        beforeEach(function () {
            fantasyXB = __1.fromPlan(function (intent$) {
                return {
                    update$: intent$.map(function (intent) {
                        switch (intent.type) {
                            case 'double':
                                return function (state) { return ({ count: state.count * 2 }); };
                            case 'half':
                                return function (state) { return ({ count: state.count / 2 }); };
                            default:
                                return function (state) { return state; };
                        }
                    }),
                    actions: {
                        double: function () { return ({ type: 'double' }); },
                        half: function () { return ({ type: 'half' }); },
                    }
                };
            });
            Counter = __1.concat(fantasyX, fantasyXB).apply(CounterView);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(CounterView);
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('should able to inc,dec,half  and double', function () {
            return t
                .do([
                actions.inc,
                actions.inc,
                actions.double,
                actions.dec,
                actions.double,
                actions.inc,
                actions.half
            ])
                .collect(counter)
                .then(function (x) { return expect(x.count).toBe(3.5); });
        });
    });
    describe('combine', function () {
        var input1;
        beforeEach(function () {
            var fantasyX1 = forms_1.xinput('value0');
            var fantasyX2 = forms_1.xinput('value1');
            var View = function (props) { return (React.createElement("div", null,
                React.createElement("span", { className: "count" }, props.sum))); };
            View.defaultProps = { sum: 0, value0: 0, value1: 0 };
            Counter = __1.lift2(function (s1, s2) { return ({ sum: (s1.value0 || 0) + (s2.value1 || 0) }); })(fantasyX1, fantasyX2).apply(View);
            counterWrapper = mountx(React.createElement(Counter, null));
            counter = counterWrapper.find(Counter).getNode();
            counterView = counterWrapper.find(View);
            input1 = counterView.find('#input1');
            actions = counterView.prop('actions');
            t = new xtests_1.rx();
        });
        it('inc will + 2', function () {
            return t
                .do([
                function () { return actions.fromEvent({ type: 'change', target: { name: 'value0', value: 3 } }); },
                function () { return actions.fromEvent({ type: 'change', target: { name: 'value1', value: 10 } }); }
            ])
                .collect(counter)
                .then(function (x) { return expect(x.sum).toBe(13); });
        });
    });
});
describe('liftN', function () {
    var Fa, stub, CounterView;
    beforeEach(function () {
        var fan = [1, 2, 3, 4, 5];
        Fa = fan.map(function (i) { return __1.fromPlan(function (intent$) {
            return {
                update$: rxjs_1.Observable.of(function (state) { return ({ count: i }); }),
                actions: (_a = {},
                    _a['action' + i] = function () { return ({ type: 'action' + i }); },
                    _a)
            };
            var _a;
        }); });
        stub = jest.fn();
        CounterView = function (props) {
            stub(props);
            return (React.createElement("div", null));
        };
        CounterView.defaultProps = { count: 0 };
    });
    describe('#lift3', function () {
        beforeEach(function () {
            var Counter = __1.lift3(function (s1, s2, s3) {
                return { count: ((s1.count || 0) + (s2.count || 0) + (s3.count || 0)) };
            })(Fa[0], Fa[1], Fa[2]).apply(CounterView);
            mountx(React.createElement(Counter, null));
        });
        it('should sum all 3 FantasyX', function () {
            expect(stub.mock.calls[1][0].count).toEqual(6);
        });
    });
    describe('#lift4', function () {
        beforeEach(function () {
            var Counter = __1.lift4(function (s1, s2, s3, s4) {
                return {
                    count: ((s1.count || 0) +
                        (s2.count || 0) +
                        (s3.count || 0)) +
                        (s4.count || 0)
                };
            })(Fa[0], Fa[1], Fa[2], Fa[3]).apply(CounterView);
            mountx(React.createElement(Counter, null));
        });
        it('should sum all 3 FantasyX', function () {
            expect(stub.mock.calls[1][0].count).toEqual(10);
        });
    });
    describe('#lift5', function () {
        beforeEach(function () {
            var Counter = __1.lift5(function (s1, s2, s3, s4, s5) {
                return {
                    count: ((s1.count || 0) +
                        (s2.count || 0) +
                        (s3.count || 0) +
                        (s4.count || 0) +
                        (s5.count || 0))
                };
            })(Fa[0], Fa[1], Fa[2], Fa[3], Fa[4]).apply(CounterView);
            mountx(React.createElement(Counter, null));
        });
        it('should sum all 3 FantasyX', function () {
            expect(stub.mock.calls[1][0].count).toEqual(15);
        });
    });
});
