"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var enzyme_1 = require("enzyme");
require("@reactivex/rxjs");
var index_1 = require("../index");
var createClass = require("create-react-class");
var compose = function (f, g) { return function (x) { return f(g(x)); }; };
var CounterView = function (props) { return (React.createElement("div", { className: "counter-view" },
    React.createElement("span", { className: "count" }, props.count))); };
CounterView.defaultProps = { count: 0, overwritedProps: 'inner' };
var xcountable = index_1.x(function (intent$) {
    return {
        update$: intent$.map(function (intent) {
            switch (intent.type) {
                case 'inc':
                    return function (state) { return ({ count: state.count + 1 }); };
                case 'dec':
                    intent$.next({ type: 'changeDefaultProps', value: -3 });
                    intent$.next({ type: 'abs' });
                    return function (state) { return ({ count: state.count - 1 }); };
                case 'abs':
                    return function (state) { return ({ count: Math.abs(state.count) }); };
                case 'changeWrapperProps':
                    return function (state) { return ({
                        wrapperProps: intent.value,
                        overwritedProps: intent.value
                    }); };
                case 'changeDefaultProps':
                    return function (state) { return ({ count: intent.value }); };
                case 'exception':
                    throw new Error('exception!!!');
                default:
                    return function (state) { return state; };
            }
        }),
        actions: {
            inc: function () { return ({ type: 'inc' }); },
            dec: function () { return ({ type: 'dec' }); },
            exception: function () { return ({ type: 'exception' }); },
            changeWrapperProps: function (value) { return ({ type: 'changeWrapperProps', value: value }); },
            changeDefaultProps: function (value) { return ({ type: 'changeDefaultProps', value: value }); },
        }
    };
});
var Counter = xcountable(CounterView);
var Xs = ['most', 'rx'];
var _loop_1 = function (name_1) {
    describe('X=' + name_1, function () {
        var engine, Xtest, mountx;
        beforeEach(function () {
            engine = require("../xs/" + name_1);
            Xtest = require("../xtests/" + name_1).default;
            mountx = compose(enzyme_1.mount, function (y) { return React.createFactory(index_1.X)({ x: engine }, y); });
        });
        describe('actions', function () {
            var counterWrapper, counter, t, counterView, actions;
            beforeEach(function () {
                counterWrapper = mountx(React.createElement(Counter, null));
                counter = counterWrapper.find(Counter).getNode();
                counterView = counterWrapper.find(CounterView);
                actions = counterView.prop('actions');
                t = new Xtest(counterView.props());
            });
            it('add intent to intent$ and go through sink$', function () {
                return t
                    .do([
                    actions.inc,
                    actions.inc,
                    actions.inc,
                ])
                    .collect(counter)
                    .then(function (x) { return expect(x.count).toBe(3); });
            });
            it('async action', function () {
                return t
                    .do([
                    actions.inc,
                    function () { return actions.fromEvent({ type: 'inc' }); },
                    function () { return actions.fromPromise(Promise.resolve({ type: 'inc' })); },
                ])
                    .collect(counter)
                    .then(function (state) { return expect(state.count).toBe(3); });
            });
            it('update can also generate new intent', function () {
                return t
                    .do([
                    counter.machine.actions.dec,
                ]).collect(counter)
                    .then(function (state) { return expect(state.count).toBe(2); });
            });
        });
        describe('props', function () {
            var counterWrapper, counter, t, counterView, actions;
            beforeEach(function () {
                counterWrapper = mountx(React.createElement(Counter, { count: 9 }));
                counter = counterWrapper.find(Counter).getNode();
                counterView = counterWrapper.find(CounterView);
                actions = counterView.prop('actions');
                t = new Xtest(counterView.props());
            });
            it('wrappers props will overwirte components default props', function () {
                return t
                    .do([
                    counter.machine.actions.inc,
                ]).collect(counter)
                    .then(function (state) { return expect(state.count).toBe(10); });
            });
        });
        describe('scope', function () {
            var counterWrapper, counter, t, counterView, actions;
            beforeEach(function () {
                counterWrapper = mountx(React.createElement(Counter, { count: 0, wrapperProps: 0, overwritedProps: 0 }));
                counter = counterWrapper.find(Counter);
                counterView = counterWrapper.find(CounterView);
                actions = counterView.prop('actions');
                t = new Xtest(counterView.props());
            });
            it('like scope in function, outter component change overwrited props wont works', function () {
                return t.plan(1)
                    .do([
                    function () { return actions.changeDefaultProps(19); },
                    function () { return counterWrapper.setProps({ wrapperProps: 1, overwritedProps: 1, count: 9 }); },
                ]).collect(counter.getNode())
                    .then(function () { return expect(counterView.prop('wrapperProps')).toBe(0); })
                    .then(function () { return expect(counterView.prop('overwritedProps')).toBe('inner'); })
                    .then(function () { return expect(counterView.prop('count')).toBe(19); });
            });
        });
        describe('composable', function () {
            var counterWrapper, counter, t, counterView, actions;
            var xxcountable = index_1.x(function (intent$) {
                return {
                    update$: intent$.map(function (intent) {
                        switch (intent.type) {
                            case 'inc2':
                                return function (state) { return ({ count: state.count + 2 }); };
                            case 'dec2':
                                return function (state) { return ({ count: state.count - 2 }); };
                            default:
                                return function (state) { return state; };
                        }
                    }),
                    actions: {
                        inc2: function () { return ({ type: 'inc2' }); },
                        dec2: function () { return ({ type: 'dec2' }); },
                    }
                };
            });
            var xxxcoutable = compose(xxcountable, xcountable);
            beforeEach(function () {
                Counter = xxxcoutable(CounterView);
                counterWrapper = mountx(React.createElement(Counter, null));
                counter = counterWrapper.find(Counter).getNode();
                counterView = counterWrapper.find(CounterView);
                actions = counterView.prop('actions');
                t = new Xtest(counterView.props());
            });
            it('compose xxcountable will provide actions inc2, dec2', function () {
                return t
                    .do([
                    actions.inc,
                    actions.inc2,
                    actions.dec2,
                ]).collect(counter)
                    .then(function (state) { return expect(state.count).toBe(1); });
            });
        });
        describe('ERROR', function () {
            var counterWrapper, counter, t, counterView, actions;
            beforeEach(function () {
                spyOn(console, 'error');
                counterWrapper = mountx(React.createElement(Counter, null));
                counter = counterWrapper.find(Counter).getNode();
                counterView = counterWrapper.find(CounterView);
                actions = counterView.prop('actions');
                t = new Xtest(counterView.props());
            });
            it('should recover to identity stream and log exception', function () {
                return t
                    .do([
                    actions.inc,
                    actions.exception,
                    actions.inc,
                ]).collect(counter)
                    .then(function (state) { return expect(state.count).toBe(2); })
                    .then(function () { return expect(console.error).toBeCalled; });
            });
        });
        describe('unsubscribe when component unmounted', function () {
            it('unsubscribe', function (done) {
                var Counter = index_1.x(function (intent$) {
                    return {
                        update$: intent$.map(function (intent) {
                            switch (intent.type) {
                                case 'inc':
                                    return function (state) { return ({ count: state.count + 1 }); };
                                default:
                                    return function (state) { return state; };
                            }
                        }),
                        actions: {
                            inc: function () { return ({ type: 'inc' }); }
                        }
                    };
                })(CounterView);
                var TogglableMount = createClass({
                    getInitialState: function () {
                        return {
                            mount: true
                        };
                    },
                    render: function () {
                        return this.state.mount && React.createElement(Counter, null);
                    }
                });
                spyOn(console, 'error');
                var counterWrapper = mountx(React.createElement(TogglableMount, null));
                var toggle = counterWrapper.find(TogglableMount).getNode();
                var counter = counterWrapper.find(Counter).getNode();
                setTimeout(function () { return done(); }, 3);
                new Xtest()
                    .do([
                    function () { return toggle.setState({ mount: false }); },
                    counter.machine.actions.inc,
                    counter.machine.actions.inc,
                ]).collect(counter)
                    .then(done.fail);
            });
        });
    });
};
for (var _i = 0, Xs_1 = Xs; _i < Xs_1.length; _i++) {
    var name_1 = Xs_1[_i];
    _loop_1(name_1);
}
